package com.jiayue.ssi.controller;

import cn.hutool.crypto.SmUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jiayue.ssi.annotation.InterfaceLimit;
import com.jiayue.ssi.config.SendMailUtil;
import com.jiayue.ssi.entity.SysUser;
import com.jiayue.ssi.service.SysUserService;
import com.jiayue.ssi.util.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.web.bind.annotation.*;

import javax.mail.internet.MimeMessage;
import javax.servlet.http.HttpServletRequest;

/**
 * 用户信息controller
 *
 * @author xsl
 * @version 3.0
 */
@RestController
@RequestMapping("/sysUserController" )
@Slf4j
public class SysUserController {
    @Autowired
    SysUserService sysUserService;
    @Autowired
    SendMailUtil sendMailUtil;

    /**
     * 获取用户分页信息
     *
     * @return 用户信息
     */
    @GetMapping(value = "/getAll")
    @InterfaceLimit
    public ResponseVO getAll(Integer currentPage, Integer pageSize, String username,String phonenumber,String status) {
        try {
            QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
            if(StringUtils.isNotEmpty(username)){
                wrapper.eq("username",username);
            }
            if(StringUtils.isNotEmpty(phonenumber)){
                wrapper.eq("phonenumber",phonenumber);
            }
            if(StringUtils.isNotEmpty(status)){
                wrapper.eq("status",status);
            }
            Page<SysUser> result = sysUserService.page(new Page<>(currentPage,pageSize),wrapper);
            return ResponseVO.success(result);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取所有用户异常");
            return ResponseVO.error(null);
        }
    }
    /**
     * 获取当前用户信息
     *
     * @return 用户信息
     */
    @GetMapping(value = "/getCurrentUser")
    @InterfaceLimit
    public ResponseVO getCurrentUser() {
        try {
            SysUser sysUser = SecurityContextUtil.getSysUser();
            return ResponseVO.success(sysUser);
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取当前用户异常");
            return ResponseVO.error(null);
        }
    }

    /**
     * 新增用户
     */
    @PostMapping
    @InterfaceLimit
    public ResponseVO add(@RequestBody SysUser user) {
        if (StringUtils.isEmpty(user.getUsername())){
            return ResponseVO.fail("用户账号不能为空!");
        }
        else if (sysUserService.queryUserName(user.getUsername())!=null){
            return ResponseVO.fail(user.getUsername() + "账号已存在！");
        }

        if (StringUtils.isEmpty(user.getPhonenumber())) {
            return ResponseVO.fail("手机号码不能为空！");
        }
        if (StringUtils.isEmpty(user.getNickname())) {
            return ResponseVO.fail("姓名不能为空！");
        }

        if (StringUtils.isEmpty(user.getMailbox())){
            return ResponseVO.fail("邮箱不能为空!");
        }
        else if (sysUserService.queryMailBox(user.getMailbox())!=null){
            return ResponseVO.fail(user.getMailbox() + "邮箱已存在！");
        }

        // 生成8位初始密码
        String randomPwd = RandomPwd.getRandomPwd(8);
        user.setPassword(SmUtil.sm3(randomPwd).toUpperCase());
        return ResponseVO.success(sysUserService.save(user));
    }

    /**
     * 初始密码发送邮箱
     */
    @PostMapping(value = "/resetPassword")
    @InterfaceLimit
    public ResponseVO resetPassword(String id) {
        if (StringUtils.isEmpty(id)){
            return ResponseVO.fail("重置密码失败!");
        }
        // id获取用户
        SysUser sysUser = sysUserService.getById(id);
        if (sysUser == null){
            return ResponseVO.fail("重置密码失败！");
        }
        // 生成8位初始密码
        String randomPwd = RandomPwd.getRandomPwd(8);
        sysUser.setPassword(SmUtil.sm3(randomPwd).toUpperCase());
        sysUser.setLastUpdatePwdTime(null);
        boolean bo = sysUserService.updateById(sysUser);
        if (!bo){
            return ResponseVO.fail("密码生成失败！");
        }
        try {
            sendMailUtil.executeSendMail(sysUser.getMailbox(),"系统登录密码","密码：" + randomPwd);
        } catch (Exception e) {
            e.printStackTrace();
            return ResponseVO.fail("发送邮箱失败");
        }
        return ResponseVO.success();
    }

    /**
     * 删除用户信息
     */
    @PostMapping(value = "/delUser")
    public ResponseVO delete(String id) {
        if (StringUtils.isEmpty(id)){
            return ResponseVO.fail("id不能为空!");
        }
        try {
            boolean bo = sysUserService.removeUserById(Integer.parseInt(id));
            if (bo){
                return ResponseVO.success("删除用户信息成功");
            }
            else{
                log.error("删除用户信息失败");
                return ResponseVO.fail("删除用户信息失败");
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("删除用户信息异常");
            return ResponseVO.error(e);
        }
    }

    /**
     * 更新用户信息
     *
     * @param sysUser 参数
     * @return 执行结果
     */
    @PutMapping(value = "sysUser/")
    public ResponseVO update(@RequestBody SysUser sysUser) {
        try {
            boolean bo = sysUserService.updateUser(sysUser);
            if (bo){
                return ResponseVO.success("修改用户信息成功");
            }
            else{
                log.error("更新用户信息失败");
                return ResponseVO.fail();
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("更新用户信息异常");
            return ResponseVO.fail();
        }
    }


}
